package Catalyst::Plugin::Authentication::PasswordChangeDetection;
{
  $Catalyst::Plugin::Authentication::PasswordChangeDetection::VERSION = '0.02';
}

use feature ":5.10";
use Moose::Role;
use Scalar::Util qw(refaddr);

around [qw(authenticate set_authenticated)] => sub {
    my $orig = shift;
    my $c = shift;

    my $ret = $c->$orig(@_);

    if ($c->user) {
        $c->session->{__password_revision} = $c->get_password_revision;
    }

    $ret;
};

around auth_restore_user => sub {
    my $orig = shift;
    my $c = shift;

    my $user = $c->$orig(@_);

    if ($user) {
        unless ( $c->{__password_change_detection_seen}{refaddr($user)}++ ) {
            if (my $new_rev_found = $c->_check_password_revision) {
                $c->password_change_detected;
                return $c->$orig(@_);
            }
        }
    }

    $user;
};

around user_exists => sub {
    my $orig = shift;
    my $c = shift;

    my $config = $c->config->{'Plugin::Authentication::PasswordChangeDetection'};
    if ( $config->{want_also_to_check_user_exists} ) {
        $c->user;
    }

    $c->$orig(@_);
};

sub get_password_revision {
    my $c = shift;

    if ( $c->user->can('password_revision') ) {
        return $c->user->password_revision;
    }
    else {
        $c->log->debug(
            sprintf(q/Disable %s feature. $c->user->can('password_revision') is false./,
                __PACKAGE__
            )
        );
        return undef;
    }
}

sub password_change_detected {
    my $c = shift;

    $c->logout;
}

sub _check_password_revision {
    my $c = shift;

    my $password_rev = $c->get_password_revision;
    my $session_rev  = $c->session->{__password_revision} // -1;

    if (!defined($password_rev)) { # disable this feature.
    }
    else {
        my $is_new_rev = $password_rev > $session_rev ? 1 : 0;
        if ($is_new_rev) {
            return 1;
        }
    }

    return 0;
}

1;

__END__

=encoding utf-8

=head1 NAME

Catalyst::Plugin::Authentication::PasswordChangeDetection - Catalyst::Plugin::Authentication::PasswordChangeDetection 

=cut
