package TestApp::Controller::Root;

use Modern::Perl;
use Moose;
BEGIN { extends "Catalyst::Controller" }

__PACKAGE__->config(
    namespace => '',
);

sub auto :Private {
    my ($self, $c) = @_;

    state $setup_done = 0;
    unless ( $setup_done++ ) {
        $c->model('DBIC')->schema->deploy;
        $c->model('DBIC::User')->create( { username => 'user1', password => 'secret' } );
    }

    1;
}

sub index :Private {
    my ($self, $c) = @_;

    $c->res->body('i am index');
}

sub login :Local {
    my ($self, $c) = @_;

    $c->res->body(
        $c->authenticate(
            {
                username => $c->req->params->{username},
                password => $c->req->params->{password},
            }
        ) ? 'success' : 'fail'
    );
}

sub user :Local {
    my ($self, $c) = @_;

    $c->res->body( $c->user ? "yes" : "no" );
}

sub user_exists :Local {
    my ($self, $c) = @_;

    $c->res->body( $c->user_exists ? "yes" : "no" );
}

sub eval :Local {
    my ($self, $c) = @_;

    my $ret = eval $c->req->params->{code};

    if ($@) {
        $c->error(@_);
        return;
    }

    $c->res->body($ret);
}

sub set_authenticated :Local {
    my ($self, $c) = @_;

    my $is_success = 0;

    my $user = $c->find_user({username=>$c->req->params->{username}});
    if ($user) {
        $c->set_authenticated($user);
        $is_success = 1;
    }

    $c->res->body( $is_success ? 'success' : 'fail' );
}

sub logout :Local {
    my ($self, $c) = @_;

    $c->logout;
}

1;
